package main

import (
	"encoding/json"
	"fmt"
	"image"
	"image/png"
	"io/ioutil"
	"os"
	"os/exec"
	"regexp"
	"strings"
	"time"

	"github.com/pkg/errors"

	"github.com/kbinani/screenshot"
	"github.com/makiuchi-d/gozxing"
	"github.com/makiuchi-d/gozxing/qrcode"
	log "github.com/sirupsen/logrus"

	"github.com/jmoiron/sqlx"
	_ "github.com/lib/pq"
)

type QRRect struct {
	X    int `json:"x"`
	Y    int `json:"y"`
	W    int `json:"w"`
	H    int `json:"h"`
	Save int `json:"save"`
}

type DBURL struct {
	Url      string `json:"url"`
	Username string `json:"username"`
	Password string `json:"password"`
}

type Config struct {
	Qrrect QRRect `json:"qrrect"`
	Dburl  DBURL  `json:"dburl"`
}

func save(img *image.RGBA, filePath string) {
	file, err := os.Create(filePath)
	if err != nil {
		panic(err)
	}
	defer file.Close()
	png.Encode(file, img)
}

var db *sqlx.DB
var local_config Config
var errRollback = errors.New("rollback")
var err_exists = errors.New("exists")

func CreateTable(db sqlx.Execer, table string) error {
	/*_, err := db.Exec(`
	create table $1
	(
		macid varchar(20) primary key,
		keyid varchar(20) not null,
		created_at timestamp with time zone not null,
		updated_at timestamp with time zone not null
	);`, "T"+table)*/
	sql := "create table if not exists lattepanda (macid varchar(20) primary key,	keyid varchar(20) not null,	created_at timestamp with time zone not null, updated_at timestamp with time zone not null);"
	fmt.Println(sql)
	_, err := db.Exec(sql)
	if err != nil {
		fmt.Println(err)
		//return errors.New("create table")
	}
	log.WithFields(log.Fields{
		"table": table,
	}).Info("create table")

	return nil
}

func CreateDevice(db sqlx.Execer, table string, KeyID string, MacID string) error {
	now := time.Now()
	CreatedAt := now
	UpdatedAt := now
	fmt.Println(table)
	//sql := "insert into " + table + " (macid, keyid, created_at, updated_at) values (" + MacID + ",	" + KeyID + ",	" + CreatedAt + ",	" + UpdatedAt + ")"
	//fmt.Println(sql)
	//_, err := db.Exec(sql)
	_, err := db.Exec(`
			insert into lattepanda (
				macid,
				keyid,
				created_at,
				updated_at
			) values ($1, $2, $3, $4)`,
		MacID,
		KeyID,
		CreatedAt,
		UpdatedAt)
	//fmt.Fprintf(os.Stdout, "insert into "+table+" (macid,	keyid,	created_at,	updated_at,	) values ("+MacID+", "+KeyID+", "+CreatedAt+", "+UpdatedAt+")")
	//_, err := db.Exec("insert into "+table+" (macid,	keyid,	created_at,	updated_at,	) values ($1, $2, $3, $4)", MacID, KeyID, CreatedAt, UpdatedAt)
	if err != nil {
		fmt.Println(err)
		return errors.New("insert error")
	}

	log.WithFields(log.Fields{
		"macid": MacID,
		"keyid": KeyID,
	}).Info("device created")

	return nil
}

func Query(db sqlx.Execer, table string, MacID string) error {
	//sql := "insert into " + table + " (macid, keyid, created_at, updated_at) values (" + MacID + ",	" + KeyID + ",	" + CreatedAt + ",	" + UpdatedAt + ")"
	//fmt.Println(sql)
	//_, err := db.Exec(sql)
	rst, err := db.Exec(`select * from lattepanda where macid=$1`, MacID)
	//fmt.Fprintf(os.Stdout, "insert into "+table+" (macid,	keyid,	created_at,	updated_at,	) values ("+MacID+", "+KeyID+", "+CreatedAt+", "+UpdatedAt+")")
	//_, err := db.Exec("insert into "+table+" (macid,	keyid,	created_at,	updated_at,	) values ($1, $2, $3, $4)", MacID, KeyID, CreatedAt, UpdatedAt)
	n, _ := rst.RowsAffected()
	if err != nil {
		fmt.Println(err)
		return errors.New("query error")
	} else {
		fmt.Println(n)
	}

	log.WithFields(log.Fields{
		"macid": MacID,
	}).Info("query macid")

	if n == 1 {
		return err_exists
	} else {
		return nil
	}

}

func Transaction(db *sqlx.DB, f func(tx sqlx.Ext) error) error {
	fmt.Println("Transaction 0")
	tx, err := db.Beginx()
	if err != nil {
		return errors.Wrap(err, "storage: begin transaction error")
	}
	fmt.Println("Transaction 1")
	err = f(tx)
	fmt.Println("Transaction 2")
	if err != nil {
		if rbErr := tx.Rollback(); rbErr != nil {
			return errors.Wrap(rbErr, "storage: transaction rollback error")
		}
		return err
	}
	fmt.Println("Transaction 3")
	if err := tx.Commit(); err != nil {
		return errors.Wrap(err, "storage: transaction commit error")
	}
	fmt.Println("Transaction 4")
	return nil
}

func save2DB(table string, KeyID string, MacID string) error {
	fmt.Println(table, KeyID, MacID)
	fmt.Println(local_config.Dburl.Url, local_config.Dburl.Username, local_config.Dburl.Password)
	table = "lattepanda"
	db, err := sqlx.Open("postgres", "postgres://"+local_config.Dburl.Username+":"+local_config.Dburl.Password+"@"+local_config.Dburl.Url+"/lattepanda_db?sslmode=disable")
	defer db.Close()
	if err != nil {
		fmt.Println(err.Error())
	}
	fmt.Println("连接远程服务器成功", db)
	err = Transaction(db, func(tx sqlx.Ext) error {
		if err := CreateTable(tx, table); err != nil {
			return err
		}
		return nil
	})
	if err != nil {
		return err
	}

	err = Transaction(db, func(tx sqlx.Ext) error {
		if err := Query(tx, table, MacID); err != nil {
			return err
		}
		return nil
	})

	if err != nil && err != err_exists {
		return err
	} else if err == nil {
		err = Transaction(db, func(tx sqlx.Ext) error {
			if err := CreateDevice(tx, table, KeyID, MacID); err != nil {
				return err
			}
			return nil
		})

		if err != nil {
			return err
		}
	}
	return nil
}

func main() {
	log.WithField("name", "wanghui").Info("start.....")
	err := os.Remove("OKOKOKOKOKOKOKOK")
	if err != nil {
		fmt.Println("---delete file failed")
	}
	time.Sleep(8 * time.Second)
	config, err := ioutil.ReadFile("./config.json")
	if err != nil {
		fmt.Println(err.Error())
	} else {
		fmt.Println(string(config))
	}

	err = json.Unmarshal(config, &local_config)
	if err != nil {
		panic(err)
	}
	fmt.Println(local_config)
	fmt.Println(time.Unix(time.Now().Unix(), 0).Format("20060102"))
	var new image.Rectangle = image.Rect(local_config.Qrrect.X, local_config.Qrrect.Y,
		local_config.Qrrect.X+local_config.Qrrect.W,
		local_config.Qrrect.Y+local_config.Qrrect.H)
	qrimg, err := screenshot.CaptureRect(new)
	now := string(time.Unix(time.Now().Unix(), 0).Format("20060102"))
	if err != nil {
		panic(err)
	}
	if local_config.Qrrect.Save == 1 {
		save(qrimg, "0.QR.png")
	}

	//fi, err := os.Open("new.png")
	//defer fi.Close()
	//qrimg, _, err = image.Decode(fi)

	bmp, err := gozxing.NewBinaryBitmapFromImage(qrimg)
	if err != nil {
		fmt.Println("4", err)
		return
	}
	// decode image
	qrReader := qrcode.NewQRCodeReader()
	result, err := qrReader.Decode(bmp, nil)
	KeyID := "1234567890123"
	if err != nil {
		fmt.Println("5", err)
		panic(err)
	} else {
		fmt.Printf("%T\r\n", result.GetText())
		KeyID = result.GetText() //string(result)
	}

	MacID := "11-22-33-44-55-66"
	cmd := exec.Command("ipconfig", "/all")
	out, _ := cmd.Output()
	//macDemo, err := ioutil.ReadFile("./mac.txt")
	//if err != nil {
	//	fmt.Println(err.Error())
	//}
	//out = macDemo
	index := strings.Index(string(out), "Ethernet adapter Ethernet")
	fmt.Println("index=", index)
	if index == -1 {
		panic(errors.New("can not find mac"))
	} else {
		macStr := string(out[index : index+353])
		fmt.Println(macStr)

		reg := regexp.MustCompile(`([A-Fa-f0-9]{2}-){5}[A-Fa-f0-9]{2}`) // 查找连续的小写字母
		fmt.Printf("%q\n", reg.FindAllString(macStr, -1))

		MacID = reg.FindAllString(macStr, -1)[0]
	}

	fmt.Println("KeyIDLen=", len(KeyID), "MacLen=", len(MacID))
	//这里加入正则出错判断
	if len(KeyID) < 8 || len(MacID) != 17 {
		return
	}

	err = save2DB(now, KeyID, MacID)

	if err == nil {
		ioutil.WriteFile("OKOKOKOKOKOKOKOK", []byte("Done"), 0666)
	}
}
